/*
 *
 *				File Unit prototypes's functions.
 *	
 *	Author	: FunShip	  
 *	File	: PCFile.h
 *	Date	: 10 February 1992
 *	Revision: 9 November 1994
 *	Version	: 1.00
 *	Language: Pure-C Version 1.1
 *	Origine : France
 *  
 */


#ifndef __MYFILE__
#define __MYFILE__	__MYFILE__

/*
 * -------------------- Dfinitions des types ---------------------------
 */

typedef struct Tposte_File
        {
          void			*Ptr_donnee;		/* ptr sur les donnees */
          struct Tposte_File	*Ptr_precedent;		/* chainage arriere */
          struct Tposte_File	*Ptr_suivant;		/* chainage avant */
        } Type_FilePoste;

typedef struct
        {
          Type_FilePoste	*Ptr_tete;		/* entree sur la tete */
          Type_FilePoste	*Ptr_queue;		/* entree sur la fin  */
          unsigned long		Taille;			/* taille des donnees */
        } Type_File;

/*
 * -------------------- Prototypes des fonctions ------------------------
 */

Type_File	*CreerFile(unsigned long Size);
/*
	Alloue une nouvelle file et l'initialise  vide.
	
	Entre:	taille en octets des donnes  emfiler
	Sortie: retourne le pointeur sur la file ou NULL si echec
*/

void		KillFile(Type_File *File);
/*
	Libre tous les postes de la file si ils existent et dtruit
	l'objet file.
	
	Entre:	pointeur sur la file
	Sortie:	rien
*/

int		EstFileVide(Type_File *File);
/*
	Indique si la file est vide ou non.
	
	Entre:	pointeur sur la file
	Sortie:	retourne 1 si vide 0 sinon
*/

void		*FileSommet(Type_File *File);
/*
	Retourne un pointeur sur la zone de donne du poste en
	tete de file. Si la file est vide la valeur NULL est renvoye.
	
	Entre:	pointeur sur la file
	Sortie:	pointeur (void *) sur la donne ou NULL si file vide
*/

int		Emfiler(Type_File *File, void *Ptr_sdonnee);
/*
	Ajoute un nouvel lment dans la file si assez de mmoire.
	
	Entre:	pointeur sur la file
		pointeur sur la donne
	Sortie:	retourne 1 si ajout 0 sinon
*/

int		Defiler(Type_File *File);
/*
	Supprime le premier lment de la file. Si la file est vide
	aucune opration n'est effectue.
	
	Entre:	pointeur sur la file
	Sortie:	retourne 1 si supprime 0 sinon
*/

int		FileEchange(Type_File *File);
/*
	Echange les positions de la premire et dernire donne dans
	la file.
	Si la file est vide ou ne possde qu'une seule donne la fonction
	n'effectue aucune opration.
	
	Entre:	pointeur sur la file
	Sortie:	retourne 1 si change effectu 0 sinon
*/

int		FileTourne(Type_File *File);
/*
	Dplace la donne en tete de file vers la queue de file.
	Si la file est vide ou ne possde qu'une seule donne la fonction
	n'effectue aucune opration.
	
	Entre:	pointeur sur la file
	Sortie:	retourne 1 si dplacement effectu 0 sinon
*/

Type_FilePoste	*FileFirst(Type_File *File);
/*
	Renvoit l'adresse du poste en tee de la file. Si la file
	est vide l'adresse sera NULL.
	
	Entre:	pointeur sur la file
	Sortie:	pointeur sur le poste ou NULL
*/

Type_FilePoste	*FileNext(Type_FilePoste *Ptr_poste);
/*
	Renvoit l'adresse du poste qui suit celui dont l'adresse est
	passe en paramtre, si il existe.
	
	Entre:	pointeur sur le poste actuel
	Sortie:	pointeur sur le poste suivant ou NULL
*/

Type_FilePoste	*FilePrevious(Type_FilePoste *Ptr_poste);
/*
	Renvoit l'adresse du poste qui prcde celui dont l'adresse est
	passe en paramtre, si il existe.
	
	Entre:	pointeur sur le poste actuel
	Sortie:	pointeur sur le poste prcdent ou NULL
*/

#endif